
module contour

contains

! ==F===================================================================================================F== !
Subroutine MakeContour(vpot,row,col,fix, minrow,maxrow,numrow, mincol,maxcol,numcol, fixvalue, filename,fct)
!
!  to generate a data file for SpyGlass graphic program
!  vpot(x,y,z) - user-define function
!
   implicit none

   interface
      double precision function vpot(xx,yy,zz)
         double precision, intent(in) :: xx,yy,zz
      end function
   end interface

   integer, intent(in) :: row,col,fix,numrow,numcol
   double precision, intent(in) :: minrow,maxrow,mincol,maxcol,fixvalue
   character(len=*), intent(in) :: filename
   double precision, intent(in), optional :: fct

   double precision, parameter :: zero = 0.0
   double precision srow,scol
   double precision, dimension(3) :: crd
   double precision, dimension(numrow) :: drow
   double precision, dimension(numcol) :: dcol,dval
   integer i,j
  
   srow=(maxrow-minrow)/(numrow-1)
   scol=(maxcol-mincol)/(numcol-1)

   do i=1,numrow
     drow(i)=minrow+srow*(i-1)
   end do

   do i=1,numcol
     dcol(i)=mincol+scol*(i-1)
   end do
  
   open(2,file=filename,action='write')
   write(2,*) numrow,numcol
   write(2,*) zero,zero
   write(2,*) (drow(i),i=1,numrow)
   write(2,*) (dcol(i),i=1,numcol)
   do i=1,numrow
     do j=1,numcol
       crd(fix)=fixvalue
       crd(row)=drow(i)
       crd(col)=dcol(j)
       dval(j)=vpot(crd(1),crd(2),crd(3))
       if (present(fct)) then
         dval(j)=fct*dval(j)
       end if
     end do
     write(2,*) (dval(j),j=1,numcol)
   end do
   close(2)

end subroutine


! ==F===================================================================================================F== !
Subroutine MakeXYZ(vpot,row,col,fix, minrow,maxrow,numrow, mincol,maxcol,numcol, fixvalue, filename,fct)
!
!  to generate data file in x,y,f format 
!  vpot(x,y,z) - user-define function  
!
   implicit none
   interface
      double precision function vpot(xx,yy,zz)
         double precision, intent(in) :: xx,yy,zz
      end function
   end interface

   integer, intent(in) :: row,col,fix,numrow,numcol
   double precision, intent(in) :: minrow,maxrow,mincol,maxcol,fixvalue
   character(len=*), intent(in) :: filename
   double precision, intent(in), optional :: fct

   double precision, parameter :: zero = 0.0
   double precision srow,scol
   double precision, dimension(3) :: crd
   double precision, dimension(numrow) :: drow
   double precision, dimension(numcol) :: dcol,dval
   integer i,j
  
   srow=(maxrow-minrow)/(numrow-1)
   scol=(maxcol-mincol)/(numcol-1)

   do i=1,numrow
     drow(i)=minrow+srow*(i-1)
   end do

   do i=1,numcol
     dcol(i)=mincol+scol*(i-1)
   end do
  
   open(2,file=filename,action='write')
   do i=1,numrow
     do j=1,numcol
       crd(fix)=fixvalue
       crd(row)=drow(i)
       crd(col)=dcol(j)
       dval(j)=vpot(crd(1),crd(2),crd(3))
       if (present(fct)) then
         dval(j)=fct*dval(j)
       end if
       write(2,*) drow(i),dcol(j),dval(j)
     end do
     write(2,*)
   end do
   close(2)

end subroutine

end module
