
module curve1d_fit 
! ****************************************************************************** !
!                                                                                !
! 1D Reproducing Kernel Hilbert spaces interpolation method for potential curves !
!                                                                                !
! ****************************************************************************** !


  use kernel
  use curve1d
  implicit none
  integer, parameter,private :: max1d = 50, maxwork1d=1000

  interface fit
     module procedure fit_rkhs1d
  end interface

contains

! ==F================================================================F== !
subroutine fit_rkhs1d(this)
  implicit none
  type (curve), intent(inout) :: this
  double precision, dimension(this%npoints,this%npoints) :: A
  double precision, dimension(this%npoints) :: B
  double precision :: rcond=1.0d-12
  integer i,j, M,N,NRHS,LDA,LDB,LWORK,rank,info

  double precision, dimension(maxwork1d) :: WORK
  double precision, dimension(this%npoints) :: S

  do i=1,this%npoints
    do j=1,this%npoints
      A(i,j)=kernel_value(this%qqq,this%x(i),this%x(j))
    end do
    B(i)=this%f(i)
  end do
  M=this%npoints
  N=M
  LDA=M
  LDB=M
  NRHS=1
  call dgelss(M,N,NRHS,A,LDA,B,LDB,S,rcond,rank,Work,maxwork1d,info)
  if (info /=0) then
    write(*,*) " ... error in dgelss called from fit_rkhs1d. info = ", info, ". Stop."
  end if
  this%c(1:M)=B

end subroutine

end module


