
module fast_sum_init

! ****************************************************************** !
!                                                                    !
!  Initilization of fast summation algorithm  for RKHS               !
!  3-dim case                                                        !
!                                                                    !
! ****************************************************************** !


   use kernel
   use rkhs_func
   use fast_sum_def

   implicit none

contains

! ==F==============================================================F== !
subroutine set_low_high(nn,mm,zz,iilow,iihigh)
!
!  to find set of indexes for each mm
!
   implicit none
   integer :: nn,mm,zz,iilow,iihigh
   intent(in) :: nn,mm,zz
   intent(out) :: iilow,iihigh

   select case (mm)
     case (1)
       iilow  = 1
       iihigh = nn
     case (2)
       if (zz > 0) then
         iilow  = 1  
         iihigh = zz
       else
         iilow  = 1
         iihigh = 0
       end if
     case (3)
       ! **** in paper by Hollebeek et al this index runs from z - a typo ? ****
       iilow  = zz+1
       iihigh = nn
   end select
end subroutine


! ==F==============================================================F== !
subroutine find_sigmas(this,grid)
!
! precompute sigmas from formula (7) in Ref.
!
  implicit none
  type (fast_sum_data), intent(out) :: this
  type (rkdata), intent(in)      :: grid 

  integer :: i,m1,m2,m3,k1,k2,k3,i1,i2,i3,ilow1,ilow2,ilow3,ihigh1,ihigh2,ihigh3,z1,z2,z3
  double precision :: sigma_temp

  
  ! (1) transfer all m's, p's, and fnum's from rkhs_func to fast_sum_data

    this%n1 = grid%n1
    this%n2 = grid%n2
    this%n3 = grid%n3
    this%r1 = grid%r1
    this%r2 = grid%r2
    this%r3 = grid%r3
    this%transtype = grid%transtype

    ! coordinate N 1
    this%m1(1)      = grid%qqq(1)%m(1)
    this%m1(2)      = grid%qqq(1)%m(2)
    this%p1(:,:)    = grid%qqq(1)%p(:,:)
    this%fnum1(:,:) = grid%qqq(1)%fnum(:,:)

    ! coordinate N 2
    this%m2(1)      = grid%qqq(2)%m(1)
    this%m2(2)      = grid%qqq(2)%m(2)
    this%p2(:,:)    = grid%qqq(2)%p(:,:)
    this%fnum2(:,:) = grid%qqq(2)%fnum(:,:)

    ! coordinate N 3
    this%m3(1)      = grid%qqq(3)%m(1)
    this%m3(2)      = grid%qqq(3)%m(2)
    this%p3(:,:)    = grid%qqq(3)%p(:,:)
    this%fnum3(:,:) = grid%qqq(3)%fnum(:,:)

  ! (2) precompute sigmas


  do z1=0,this%n1
   do z2=0,this%n2
    do z3=0,this%n3

      do m1=1,3
        do m2=1,3
          do m3=1,3

             call set_low_high(this%n1,m1,z1,ilow1,ihigh1)
             call set_low_high(this%n2,m2,z2,ilow2,ihigh2)
             call set_low_high(this%n3,m3,z3,ilow3,ihigh3)

             do k1=1,mmax
               do k2=1,mmax
                 do k3=1,mmax

                    sigma_temp = 0.0d0

                    if ((ilow1 <= ihigh1).and.(ilow2 <= ihigh2).and.(ilow3 <= ihigh3)) then

                     do i1=ilow1,ihigh1
                       do i2=ilow2,ihigh2
                         do i3=ilow3,ihigh3
                           sigma_temp = sigma_temp + this%p1(m1,k1)*func_arr(this%fnum1(m1,k1),this%r1(i1)) &
                                                   * this%p2(m2,k2)*func_arr(this%fnum2(m2,k2),this%r2(i2)) &
                                                   * this%p3(m3,k3)*func_arr(this%fnum3(m3,k3),this%r3(i3)) &
                                                   * grid%alpha(i1,i2,i3)
                                                                          
                         end do
                       end do
                     end do

                    end if

                    this%zeta(z1,z2,z3)%sigma(m1,k1,m2,k2,m3,k3) = sigma_temp

                 end do
               end do
             end do

          end do
        end do
      end do

    end do
   end do
  end do

end subroutine

! ==F==============================================================F== !
subroutine write_sigmas(this,filename,ns)
!
!  to flush the whole structure into binary file
!
  implicit none
  type (fast_sum_data), intent(in) :: this
  character(len=*), intent(in) :: filename
   integer, optional, intent(in) :: ns

  integer :: nstream

  if (present(ns)) then
    nstream = ns
  else
    nstream = 2
  end if

  open(nstream,file=filename,action='write',access='sequential',form='unformatted')
  write(nstream) this
  close(nstream)

end subroutine


end module
