
module global_cutoff

! ******************************************************************* !
!                                                                     !
! cutting function to zero at too short and/or too long distances     !
!                                                                     !
! ******************************************************************* !
   implicit none

   type global_cutoff_func
       double precision :: r1min, r2min, r3min
       double precision :: r1max, r2max, r3max
       double precision :: g1min, g2min, g3min
       double precision :: g1max, g2max, g3max
   end type


contains

! ==F===============================================================F== !
Subroutine clear_cutoff(this)
   implicit none
   type (global_cutoff_func), intent(inout) :: this

   this%r1min = -1.0
   this%r2min = -1.0
   this%r3min = -1.0

   this%r1max = -1.0
   this%r2max = -1.0
   this%r3max = -1.0

End subroutine


! ==F===============================================================F== !
Subroutine set_short_cutoff(this,r1,r2,r3,g1,g2,g3)
   implicit none
   type (global_cutoff_func), intent(inout) :: this
   double precision, intent(in) :: r1,r2,r3,g1,g2,g3

   this%r1min = r1
   this%r2min = r2
   this%r3min = r3

   this%g1min = g1
   this%g2min = g2
   this%g3min = g3
  
End subroutine

! ==F===============================================================F== !
Subroutine set_long_cutoff(this,r1,r2,r3,g1,g2,g3)
   implicit none
   type (global_cutoff_func), intent(inout) :: this
   double precision, intent(in) :: r1,r2,r3,g1,g2,g3

   this%r1max = r1
   this%r2max = r2
   this%r3max = r3

   this%g1max = g1
   this%g2max = g2
   this%g3max = g3
  
End subroutine

! ==F===============================================================F== !
function short_cutoff(rmin,ro,r) result (f)
    implicit none
    double precision, intent(in) :: rmin, ro, r
    double precision :: f

    if (rmin < 0.0d0) then
       f = 1.0d0
    else
       f =  0.5*(tanh(ro*(r-rmin))+1.0)
    end if

end function

! ==F===============================================================F== !
function long_cutoff(rmax,ro,r) result(f)
    implicit none
    double precision, intent(in) :: rmax, ro, r
    double precision :: f

    if (rmax < 0.0d0) then
       f = 1.0d0
    else
       f = 0.5 * ( 1.0d0-tanh(ro*(r-rmax)) )
    end if

end function

! ==F===============================================================F== !
function eval_cutoff(this,r1,r2,r3) result (f)
  implicit none
  type (global_cutoff_func), intent(in) :: this
  double precision, intent(in) :: r1, r2, r3
  double precision :: f

  f =    short_cutoff(this%r1min,this%g1min,r1) &
       * short_cutoff(this%r2min,this%g2min,r2) &
       * short_cutoff(this%r3min,this%g3min,r3) &
       *  long_cutoff(this%r1max,this%g1max,r1) &
       *  long_cutoff(this%r2max,this%g2max,r2) &
       *  long_cutoff(this%r3max,this%g3max,r3) 

end function


! ==F===============================================================F== !
function exp_cutoff_1d(rmin,alp,r) result (f)
    implicit none
    double precision, intent(in) :: rmin, alp, r 
    double precision :: f

    if (r <= rmin) then
       f = 0.0d0
    else
       f = exp(alp/(rmin-r))
    end if

end function


! ==F===============================================================F== !
function exp_cutoff(this,r1,r2,r3) result (f)
  implicit none
  type (global_cutoff_func), intent(in) :: this
  double precision, intent(in) :: r1, r2, r3
  double precision :: f

  f =   exp_cutoff_1d(this%r1min,this%g1min,r1) &
      * exp_cutoff_1d(this%r2min,this%g2min,r2) &
      * exp_cutoff_1d(this%r3min,this%g3min,r3) 

  
end function


end module
