
module hgbr2_surf_def

! Definitions and read/write procedures

! ****************************************************************** !
!                                                                    !
!     icMRCI(Q)/CSB + SO corr. global potential of HgBr2             !
!                                                                    !
! Combined from four separate pieces:                                !
!                                                                    !
! 1) abstraction reaction, HgBr + Br -> Hg+Br2, RKHS interpolation   !
! 2) insertion reaction, Hg + Br2 -> BrHgBr, RKHS interpolation      !
! 3) recombination reaction, Br + HgBr -> BrHgBr, RKHS interpolation !
! 4) near-equlibrium PES of BrHgBr, analytical fit                   !
!                                                                    !
! r1 = r(HgBr), r2=r(HgBr), r3 = r(BrBr)                             !
!                                                                    !
! ****************************************************************** !
   
   use global_cutoff
   use surfitpol
   use curve1d
   use fast_sum_def

   implicit none

   type hgbr2_surface 
      character (len=60)        ::  info
      double precision          ::  onebody
      type (curve)              ::  hgbr,brbr
      type (fast_sum_data)      ::  rec,ins,abs 
      type (surfpol)            ::  gmin
      double precision          ::  r1eq, r2eq, alp_eq, &
                                    dlt1eq, dlt2eq, dlta_eq, &
                                    gamma_eq, ro_eq, singcorr, &
                                    beta_sw, alp_sw, beta_g, alp_g, &
                                    r1max, r2max, r3max, &
                                    g1max, g2max, g3max 

    end type

    type (hgbr2_surface)        :: GS

contains

! == F ===========================================================================
Subroutine write_out(this,filename,ns)
  implicit none
  type (hgbr2_surface), intent(in) :: this
  character (len=*), intent(in) :: filename
  integer, optional, intent(in) :: ns

  integer :: nstream

  if (present(ns)) then
    nstream = ns
  else
    nstream = 2
  end if

  open(nstream,file=filename,action='write',access='sequential',form='unformatted')
  write(nstream) this
  close(nstream)
  
end subroutine

! == F ===========================================================================
Subroutine read_in(this,filename,ns)
  implicit none
  type (hgbr2_surface), intent(out) :: this
  character (len=*), intent(in) :: filename
  integer, optional, intent(in) :: ns

  integer :: nstream

  if (present(ns)) then
    nstream = ns
  else
    nstream = 2
  end if

  open(nstream,file=filename,action='read',access='sequential',form='unformatted')
  read(nstream) this
  close(nstream)

end subroutine

end module
